"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _jsonschema = require("jsonschema");

var _SchemaField2 = require("./fields/SchemaField");

var _SchemaField3 = _interopRequireDefault(_SchemaField2);

var _TitleField2 = require("./fields/TitleField");

var _TitleField3 = _interopRequireDefault(_TitleField2);

var _utils = require("../utils");

var _ErrorList = require("./ErrorList");

var _ErrorList2 = _interopRequireDefault(_ErrorList);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Form = function (_Component) {
  _inherits(Form, _Component);

  function Form(props) {
    _classCallCheck(this, Form);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Form).call(this, props));

    _this.onChange = function (formData) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? { validate: false } : arguments[1];

      var mustValidate = _this.props.liveValidate || options.validate;
      var state = void 0;
      if (!mustValidate) {
        state = { status: "editing", formData: formData };
      } else {
        var errors = _this.validate(formData);
        var errorSchema = (0, _utils.toErrorSchema)(errors);
        state = { status: "editing", formData: formData, errors: errors, errorSchema: errorSchema };
      }
      (0, _utils.setState)(_this, state, function () {
        if (_this.props.onChange) {
          _this.props.onChange(_this.state);
        }
      });
    };

    _this.onSubmit = function (event) {
      event.preventDefault();
      _this.setState({ status: "submitted" });
      var errors = _this.validate(_this.state.formData);
      if (Object.keys(errors).length > 0) {
        var errorSchema = (0, _utils.toErrorSchema)(errors);
        (0, _utils.setState)(_this, { errors: errors, errorSchema: errorSchema }, function () {
          if (_this.props.onError) {
            _this.props.onError(errors);
          } else {
            console.error("Form validation failed", errors);
          }
        });
        return;
      } else if (_this.props.onSubmit) {
        _this.props.onSubmit(_this.state);
      }
      _this.setState({ status: "initial", errors: [], errorSchema: {} });
    };

    _this.state = _this.getStateFromProps(props);
    return _this;
  }

  _createClass(Form, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState(this.getStateFromProps(nextProps));
    }
  }, {
    key: "getStateFromProps",
    value: function getStateFromProps(props) {
      var state = this.state || {};
      var schema = "schema" in props ? props.schema : this.props.schema;
      var uiSchema = "uiSchema" in props ? props.uiSchema : this.props.uiSchema;
      var edit = typeof props.formData !== "undefined";
      var liveValidate = props.liveValidate || this.props.liveValidate;
      var mustValidate = edit && liveValidate;
      var definitions = schema.definitions;

      var formData = (0, _utils.getDefaultFormState)(schema, props.formData, definitions);
      var previousErrors = state.errors || [];
      var previousErrorSchema = state.errorSchema || {};
      var errors = mustValidate ? this.validate(formData, schema) : previousErrors;
      var errorSchema = mustValidate ? (0, _utils.toErrorSchema)(errors) : previousErrorSchema;
      var idSchema = (0, _utils.toIdSchema)(schema, uiSchema["ui:rootFieldId"], definitions);
      return { status: "initial", formData: formData, edit: edit, errors: errors, errorSchema: errorSchema, idSchema: idSchema };
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps, nextState) {
      return (0, _utils.shouldRender)(this, nextProps, nextState);
    }
  }, {
    key: "validate",
    value: function validate(formData, schema) {
      var validator = new _jsonschema.Validator();
      return validator.validate(formData, schema || this.props.schema).errors;
    }
  }, {
    key: "renderErrors",
    value: function renderErrors() {
      var _state = this.state;
      var status = _state.status;
      var errors = _state.errors;

      if (status !== "editing" && errors.length) {
        return _react2.default.createElement(_ErrorList2.default, { errors: errors });
      }
      return null;
    }
  }, {
    key: "getRegistry",
    value: function getRegistry() {
      // For BC, accept passed SchemaField and TitleField props and pass them to
      // the "fields" registry one.
      var _SchemaField = this.props.SchemaField || _SchemaField3.default;
      var _TitleField = this.props.TitleField || _TitleField3.default;
      var fields = Object.assign({
        SchemaField: _SchemaField,
        TitleField: _TitleField
      }, this.props.fields);
      return {
        fields: fields,
        widgets: this.props.widgets || {},
        definitions: this.props.schema.definitions || {}
      };
    }
  }, {
    key: "render",
    value: function render() {
      var _props = this.props;
      var children = _props.children;
      var schema = _props.schema;
      var uiSchema = _props.uiSchema;
      var safeRenderCompletion = _props.safeRenderCompletion;
      var _state2 = this.state;
      var formData = _state2.formData;
      var errorSchema = _state2.errorSchema;
      var idSchema = _state2.idSchema;

      var registry = this.getRegistry();
      var _SchemaField = registry.fields.SchemaField;
      return _react2.default.createElement(
        "form",
        { className: "rjsf", onSubmit: this.onSubmit },
        this.renderErrors(),
        _react2.default.createElement(_SchemaField, {
          schema: schema,
          uiSchema: uiSchema,
          errorSchema: errorSchema,
          idSchema: idSchema,
          formData: formData,
          onChange: this.onChange,
          registry: registry,
          safeRenderCompletion: safeRenderCompletion }),
        children ? children : _react2.default.createElement(
          "p",
          null,
          _react2.default.createElement(
            "button",
            { type: "submit", className: "btn btn-info" },
            "Submit"
          )
        )
      );
    }
  }]);

  return Form;
}(_react.Component);

Form.defaultProps = {
  uiSchema: {},
  liveValidate: false,
  safeRenderCompletion: false
};
exports.default = Form;


if (process.env.NODE_ENV !== "production") {
  Form.propTypes = {
    schema: _react.PropTypes.object.isRequired,
    uiSchema: _react.PropTypes.object,
    formData: _react.PropTypes.any,
    widgets: _react.PropTypes.objectOf(_react.PropTypes.func),
    fields: _react.PropTypes.objectOf(_react.PropTypes.func),
    onChange: _react.PropTypes.func,
    onError: _react.PropTypes.func,
    onSubmit: _react.PropTypes.func,
    liveValidate: _react.PropTypes.bool,
    safeRenderCompletion: _react.PropTypes.bool
  };
}

exports.default = Form;