"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.getDefaultRegistry = getDefaultRegistry;
exports.defaultFieldValue = defaultFieldValue;
exports.getAlternativeWidget = getAlternativeWidget;
exports.getDefaultFormState = getDefaultFormState;
exports.mergeObjects = mergeObjects;
exports.asNumber = asNumber;
exports.orderProperties = orderProperties;
exports.isMultiSelect = isMultiSelect;
exports.isFixedItems = isFixedItems;
exports.allowAdditionalItems = allowAdditionalItems;
exports.optionsList = optionsList;
exports.retrieveSchema = retrieveSchema;
exports.shouldRender = shouldRender;
exports.toErrorSchema = toErrorSchema;
exports.toIdSchema = toIdSchema;
exports.parseDateString = parseDateString;
exports.toDateString = toDateString;
exports.pad = pad;
exports.setState = setState;

var _deeper = require("deeper");

var _deeper2 = _interopRequireDefault(_deeper);

var _TitleField = require("./components/fields/TitleField");

var _TitleField2 = _interopRequireDefault(_TitleField);

var _PasswordWidget = require("./components/widgets/PasswordWidget");

var _PasswordWidget2 = _interopRequireDefault(_PasswordWidget);

var _RadioWidget = require("./components/widgets/RadioWidget");

var _RadioWidget2 = _interopRequireDefault(_RadioWidget);

var _UpDownWidget = require("./components/widgets/UpDownWidget");

var _UpDownWidget2 = _interopRequireDefault(_UpDownWidget);

var _RangeWidget = require("./components/widgets/RangeWidget");

var _RangeWidget2 = _interopRequireDefault(_RangeWidget);

var _SelectWidget = require("./components/widgets/SelectWidget");

var _SelectWidget2 = _interopRequireDefault(_SelectWidget);

var _TextWidget = require("./components/widgets/TextWidget");

var _TextWidget2 = _interopRequireDefault(_TextWidget);

var _DateWidget = require("./components/widgets/DateWidget");

var _DateWidget2 = _interopRequireDefault(_DateWidget);

var _DateTimeWidget = require("./components/widgets/DateTimeWidget");

var _DateTimeWidget2 = _interopRequireDefault(_DateTimeWidget);

var _AltDateWidget = require("./components/widgets/AltDateWidget");

var _AltDateWidget2 = _interopRequireDefault(_AltDateWidget);

var _AltDateTimeWidget = require("./components/widgets/AltDateTimeWidget");

var _AltDateTimeWidget2 = _interopRequireDefault(_AltDateTimeWidget);

var _EmailWidget = require("./components/widgets/EmailWidget");

var _EmailWidget2 = _interopRequireDefault(_EmailWidget);

var _URLWidget = require("./components/widgets/URLWidget");

var _URLWidget2 = _interopRequireDefault(_URLWidget);

var _TextareaWidget = require("./components/widgets/TextareaWidget");

var _TextareaWidget2 = _interopRequireDefault(_TextareaWidget);

var _HiddenWidget = require("./components/widgets/HiddenWidget");

var _HiddenWidget2 = _interopRequireDefault(_HiddenWidget);

var _ColorWidget = require("./components/widgets/ColorWidget");

var _ColorWidget2 = _interopRequireDefault(_ColorWidget);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var RE_ERROR_ARRAY_PATH = /\[\d+]/g;

var altWidgetMap = {
  boolean: {
    radio: _RadioWidget2.default,
    select: _SelectWidget2.default,
    hidden: _HiddenWidget2.default
  },
  string: {
    password: _PasswordWidget2.default,
    radio: _RadioWidget2.default,
    select: _SelectWidget2.default,
    textarea: _TextareaWidget2.default,
    hidden: _HiddenWidget2.default,
    date: _DateWidget2.default,
    datetime: _DateTimeWidget2.default,
    "alt-date": _AltDateWidget2.default,
    "alt-datetime": _AltDateTimeWidget2.default,
    color: _ColorWidget2.default
  },
  number: {
    updown: _UpDownWidget2.default,
    range: _RangeWidget2.default,
    hidden: _HiddenWidget2.default
  },
  integer: {
    updown: _UpDownWidget2.default,
    range: _RangeWidget2.default,
    hidden: _HiddenWidget2.default
  }
};

var stringFormatWidgets = {
  "date-time": _DateTimeWidget2.default,
  "date": _DateWidget2.default,
  "email": _EmailWidget2.default,
  "hostname": _TextWidget2.default,
  "ipv4": _TextWidget2.default,
  "ipv6": _TextWidget2.default,
  "uri": _URLWidget2.default
};

function getDefaultRegistry() {
  return {
    fields: {
      // Prevent a bug where SchemaField is undefined when imported via Babel.
      // This seems to have been introduced when upgrading React from 0.14 to to
      // 15.0, which now seems to prevent cyclic references of exported
      // components.
      // Investigation hint: getDefaultRegistry is called from within
      // SchemaField itself.
      SchemaField: require("./components/fields/SchemaField").default,
      TitleField: _TitleField2.default
    },
    widgets: {},
    definitions: {}
  };
}

function defaultFieldValue(formData, schema) {
  return typeof formData === "undefined" ? schema.default : formData;
}

function getAlternativeWidget(schema, widget) {
  var registeredWidgets = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var type = schema.type;
  var format = schema.format;

  if (typeof widget === "function") {
    return widget;
  }
  if (typeof widget !== "string") {
    throw new Error("Unsupported widget definition: " + (typeof widget === "undefined" ? "undefined" : _typeof(widget)));
  }
  if (widget in registeredWidgets) {
    return registeredWidgets[widget];
  }
  if (!altWidgetMap.hasOwnProperty(type)) {
    throw new Error("No alternative widget for type " + type);
  }
  if (altWidgetMap[type].hasOwnProperty(widget)) {
    return altWidgetMap[type][widget];
  }
  if (type === "string" && stringFormatWidgets.hasOwnProperty(format)) {
    return stringFormatWidgets[format];
  }
  var info = type === "string" && format ? "/" + format : "";
  throw new Error("No alternative widget \"" + widget + "\" for type " + type + info);
}

function computeDefaults(schema, parentDefaults) {
  var definitions = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  // Compute the defaults recursively: give highest priority to deepest nodes.
  var defaults = parentDefaults;
  if (isObject(defaults) && isObject(schema.default)) {
    // For object defaults, only override parent defaults that are defined in
    // schema.default.
    defaults = mergeObjects(defaults, schema.default);
  } else if ("default" in schema) {
    // Use schema defaults for this node.
    defaults = schema.default;
  } else if ("enum" in schema && Array.isArray(schema.enum)) {
    // For enum with no defined default, select the first entry.
    defaults = schema.enum[0];
  } else if ("$ref" in schema) {
    // Use referenced schema defaults for this node.
    var refSchema = findSchemaDefinition(schema.$ref, definitions);
    return computeDefaults(refSchema, defaults, definitions);
  } else if (isFixedItems(schema)) {
    defaults = schema.items.map(function (itemSchema) {
      return computeDefaults(itemSchema, undefined, definitions);
    });
  }
  // Not defaults defined for this node, fallback to generic typed ones.
  if (typeof defaults === "undefined") {
    defaults = schema.default;
  }
  // We need to recur for object schema inner default values.
  if (schema.type === "object") {
    return Object.keys(schema.properties).reduce(function (acc, key) {
      // Compute the defaults for this node, with the parent defaults we might
      // have from a previous run: defaults[key].
      acc[key] = computeDefaults(schema.properties[key], (defaults || {})[key], definitions);
      return acc;
    }, {});
  }
  return defaults;
}

function getDefaultFormState(_schema, formData) {
  var definitions = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  if (!isObject(_schema)) {
    throw new Error("Invalid schema: " + _schema);
  }
  var schema = retrieveSchema(_schema, definitions);
  var defaults = computeDefaults(schema, _schema.default, definitions);
  if (typeof formData === "undefined") {
    // No form data? Use schema defaults.
    return defaults;
  }
  if (isObject(formData)) {
    // Override schema defaults with form data.
    return mergeObjects(defaults, formData);
  }
  return formData || defaults;
}

function isObject(thing) {
  return (typeof thing === "undefined" ? "undefined" : _typeof(thing)) === "object" && thing !== null && !Array.isArray(thing);
}

function mergeObjects(obj1, obj2) {
  // Recursively merge deeply nested objects.
  var acc = Object.assign({}, obj1); // Prevent mutation of source object.
  return Object.keys(obj2).reduce(function (acc, key) {
    var right = obj2[key];
    if (obj1.hasOwnProperty(key) && isObject(right)) {
      acc[key] = mergeObjects(obj1[key], right);
    } else {
      acc[key] = right;
    }
    return acc;
  }, acc);
}

function asNumber(value) {
  if (/\.$/.test(value)) {
    // "3." can't really be considered a number even if it parses in js. The
    // user is most likely entering a float.
    return value;
  }
  var n = Number(value);
  var valid = typeof n === "number" && !Number.isNaN(n);
  return valid ? n : value;
}

function orderProperties(properties, order) {
  if (!Array.isArray(order)) {
    return properties;
  }
  if (order.length !== properties.length) {
    throw new Error("uiSchema order list length should match object properties length");
  }
  var fingerprint = function fingerprint(arr) {
    return [].slice.call(arr).sort().toString();
  };
  if (fingerprint(order) !== fingerprint(properties)) {
    throw new Error("uiSchema order list does not match object properties list");
  }
  return order;
}

function isMultiSelect(schema) {
  return Array.isArray(schema.items.enum) && schema.uniqueItems;
}

function isFixedItems(schema) {
  return Array.isArray(schema.items) && schema.items.length > 0 && schema.items.every(function (item) {
    return isObject(item);
  });
}

function allowAdditionalItems(schema) {
  if (schema.additionalItems === true) {
    console.warn("additionalItems=true is currently not supported");
  }
  return isObject(schema.additionalItems);
}

function optionsList(schema) {
  return schema.enum.map(function (value, i) {
    var label = schema.enumNames && schema.enumNames[i] || String(value);
    return { label: label, value: value };
  });
}

function findSchemaDefinition($ref) {
  var definitions = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  // Extract and use the referenced definition if we have it.
  var match = /#\/definitions\/(.*)$/.exec($ref);
  if (match && match[1] && definitions.hasOwnProperty(match[1])) {
    return definitions[match[1]];
  }
  // No matching definition found, that's an error (bogus schema?)
  throw new Error("Could not find a definition for " + $ref + ".");
}

function retrieveSchema(schema) {
  var definitions = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  // No $ref attribute found, returning the original schema.
  if (!schema.hasOwnProperty("$ref")) {
    return schema;
  }
  // Retrieve the referenced schema definition.
  var $refSchema = findSchemaDefinition(schema.$ref, definitions);
  // Drop the $ref property of the source schema.
  var $ref = schema.$ref;

  var localSchema = _objectWithoutProperties(schema, ["$ref"]); // eslint-disable-line no-unused-vars
  // Update referenced schema definition with local schema properties.


  return _extends({}, $refSchema, localSchema);
}

function shouldRender(comp, nextProps, nextState) {
  return !(0, _deeper2.default)(comp.props, nextProps) || !(0, _deeper2.default)(comp.state, nextState);
}

function errorPropertyToPath(property) {
  // Parse array indices, eg. "instance.level1.level2[2].level3"
  // => ["instance", "level1", "level2", 2, "level3"]
  return property.split(".").reduce(function (path, node) {
    var match = node.match(RE_ERROR_ARRAY_PATH);
    if (match) {
      var nodeName = node.slice(0, node.indexOf("["));
      var indices = match.map(function (str) {
        return parseInt(str.slice(1, -1), 10);
      });
      path = path.concat(nodeName, indices);
    } else {
      path.push(node);
    }
    return path;
  }, []);
}

function toErrorSchema(errors) {
  // Transforms a jsonschema validation errors list:
  // [
  //   {property: "instance.level1.level2[2].level3", message: "err a"},
  //   {property: "instance.level1.level2[2].level3", message: "err b"},
  //   {property: "instance.level1.level2[4].level3", message: "err b"},
  // ]
  // Into an error tree:
  // {
  //   level1: {
  //     level2: {
  //       2: {level3: {errors: ["err a", "err b"]}},
  //       4: {level3: {errors: ["err b"]}},
  //     }
  //   }
  // };
  if (!errors.length) {
    return {};
  }
  return errors.reduce(function (errorSchema, error) {
    var property = error.property;
    var message = error.message;

    var path = errorPropertyToPath(property);
    var parent = errorSchema;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = path.slice(1)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var segment = _step.value;

        if (!(segment in parent)) {
          parent[segment] = {};
        }
        parent = parent[segment];
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    if (Array.isArray(parent.errors)) {
      parent.errors = parent.errors.concat(message);
    } else {
      parent.errors = [message];
    }
    return errorSchema;
  }, {});
}

function toIdSchema(schema, id, definitions) {
  var idSchema = { id: id || "root" };
  if ("$ref" in schema) {
    var _schema = retrieveSchema(schema, definitions);
    return toIdSchema(_schema, id, definitions);
  }
  if ("items" in schema) {
    return toIdSchema(schema.items, id, definitions);
  }
  if (schema.type !== "object") {
    return idSchema;
  }
  for (var name in schema.properties || {}) {
    var field = schema.properties[name];
    var fieldId = idSchema.id + "_" + name;
    idSchema[name] = toIdSchema(field, fieldId, definitions);
  }
  return idSchema;
}

function parseDateString(dateString) {
  var includeTime = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

  if (!dateString) {
    return {
      year: -1,
      month: -1,
      day: -1,
      hour: includeTime ? -1 : 0,
      minute: includeTime ? -1 : 0,
      second: includeTime ? -1 : 0
    };
  }
  var date = new Date(dateString);
  if (Number.isNaN(date.getTime())) {
    throw new Error("Unable to parse date " + dateString);
  }
  return {
    year: date.getUTCFullYear(),
    month: date.getUTCMonth() + 1, // oh you, javascript.
    day: date.getUTCDate(),
    hour: includeTime ? date.getUTCHours() : 0,
    minute: includeTime ? date.getUTCMinutes() : 0,
    second: includeTime ? date.getUTCSeconds() : 0
  };
}

function toDateString(_ref) {
  var year = _ref.year;
  var month = _ref.month;
  var day = _ref.day;
  var _ref$hour = _ref.hour;
  var hour = _ref$hour === undefined ? 0 : _ref$hour;
  var _ref$minute = _ref.minute;
  var minute = _ref$minute === undefined ? 0 : _ref$minute;
  var _ref$second = _ref.second;
  var second = _ref$second === undefined ? 0 : _ref$second;
  var time = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

  var utcTime = Date.UTC(year, month - 1, day, hour, minute, second);
  var datetime = new Date(utcTime).toJSON();
  return time ? datetime : datetime.slice(0, 10);
}

function pad(num, size) {
  var s = String(num);
  while (s.length < size) {
    s = "0" + s;
  }
  return s;
}

function setState(instance, state, callback) {
  var safeRenderCompletion = instance.props.safeRenderCompletion;

  if (safeRenderCompletion) {
    instance.setState(state, callback);
  } else {
    instance.setState(state);
    setImmediate(callback);
  }
}